@@ -50,10 +50,7 @@ farming.register_plant("farming:cotton", {
 	fertility = {"grassland", "desert"}
 })
 
-minetest.register_craftitem("farming:string", {
-	description = "String",
-	inventory_image = "farming_cotton.png",
-})
+minetest.register_alias("farming:string", "farming:cotton")
 
 minetest.register_craft({
 	output = "wool:white",
