@@ -27,7 +27,7 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #include "object_properties.h"
 
 ServerActiveObject* createItemSAO(ServerEnvironment *env, v3f pos,
-		const std::string itemstring);
+                                  const std::string &itemstring);
 
 /*
 	LuaEntitySAO needs some internals exposed.
@@ -37,7 +37,7 @@ class LuaEntitySAO : public ServerActiveObject
 {
 public:
 	LuaEntitySAO(ServerEnvironment *env, v3f pos,
-			const std::string &name, const std::string &state);
+	             const std::string &name, const std::string &state);
 	~LuaEntitySAO();
 	u8 getType() const
 	{ return ACTIVEOBJECT_TYPE_LUAENTITY; }
@@ -79,6 +79,7 @@ public:
 			bool select_horiz_by_yawpitch);
 	std::string getName();
 	bool getCollisionBox(aabb3f *toset);
+	bool collideWithObjects();
 private:
 	std::string getPropertyPacket();
 	void sendPosition(bool do_interpolate, bool is_movement_end);
@@ -121,6 +122,36 @@ private:
 	PlayerSAO needs some internals exposed.
 */
 
+class LagPool
+{
+	float m_pool;
+	float m_max;
+public:
+	LagPool(): m_pool(15), m_max(15)
+	{}
+	void setMax(float new_max)
+	{
+		m_max = new_max;
+		if(m_pool > new_max)
+			m_pool = new_max;
+	}
+	void add(float dtime)
+	{
+		m_pool -= dtime;
+		if(m_pool < 0)
+			m_pool = 0;
+	}
+	bool grab(float dtime)
+	{
+		if(dtime <= 0)
+			return true;
+		if(m_pool + dtime > m_max)
+			return false;
+		m_pool += dtime;
+		return true;
+	}
+};
+
 class PlayerSAO : public ServerActiveObject
 {
 public:
@@ -140,7 +171,6 @@ public:
 	void addedToEnvironment(u32 dtime_s);
 	void removingFromEnvironment();
 	bool isStaticAllowed() const;
-	bool unlimitedTransferDistance() const;
 	std::string getClientInitializationData(u16 protocol_version);
 	std::string getStaticData();
 	bool isAttached();
@@ -162,7 +192,9 @@ public:
 	void rightClick(ServerActiveObject *clicker);
 	s16 getHP() const;
 	void setHP(s16 hp);
-	
+	s16 readDamage();
+	u16 getBreath() const;
+	void setBreath(u16 breath);
 	void setArmorGroups(const ItemGroupList &armor_groups);
 	void setAnimation(v2f frame_range, float frame_speed, float frame_blend);
 	void setBonePosition(std::string bone, v3f position, v3f rotation);
@@ -226,6 +258,12 @@ public:
 	{
 		m_nocheat_dig_pos = v3s16(32767, 32767, 32767);
 	}
+	LagPool& getDigPool()
+	{
+		return m_dig_pool;
+	}
+	// Returns true if cheated
+	bool checkMovementCheat();
 
 	// Other
 
@@ -237,6 +275,7 @@ public:
 	}
 
 	bool getCollisionBox(aabb3f *toset);
+	bool collideWithObjects();
 
 private:
 	std::string getPropertyPacket();
@@ -244,10 +283,12 @@ private:
 	Player *m_player;
 	u16 m_peer_id;
 	Inventory *m_inventory;
+	s16 m_damage;
 
 	// Cheat prevention
+	LagPool m_dig_pool;
+	LagPool m_move_pool;
 	v3f m_last_good_position;
-	float m_last_good_position_age;
 	float m_time_from_last_punch;
 	v3s16 m_nocheat_dig_pos;
 	float m_nocheat_dig_time;
@@ -282,11 +323,14 @@ public:
 	bool m_moved;
 	bool m_inventory_not_sent;
 	bool m_hp_not_sent;
+	bool m_breath_not_sent;
 	bool m_wielded_item_not_sent;
 
 	float m_physics_override_speed;
 	float m_physics_override_jump;
 	float m_physics_override_gravity;
+	bool m_physics_override_sneak;
+	bool m_physics_override_sneak_glitch;
 	bool m_physics_override_sent;
 };
 
