@@ -33,10 +33,10 @@ public:
 	Database_Dummy(ServerMap *map);
 	virtual void beginSave();
 	virtual void endSave();
-        virtual void saveBlock(MapBlock *block);
-        virtual MapBlock* loadBlock(v3s16 blockpos);
-        virtual void listAllLoadableBlocks(std::list<v3s16> &dst);
-        virtual int Initialized(void);
+	virtual bool saveBlock(v3s16 blockpos, std::string &data);
+	virtual std::string loadBlock(v3s16 blockpos);
+	virtual void listAllLoadableBlocks(std::list<v3s16> &dst);
+	virtual int Initialized(void);
 	~Database_Dummy();
 private:
 	ServerMap *srvmap;
