@@ -23,8 +23,8 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #include <map>
 #include "irr_v3d.h"
 #include "util/container.h"
-#include "map.h" // for ManualMapVoxelManipulator
 #include "mapgen.h" // for MapgenParams
+#include "map.h"
 
 #define MGPARAMS_SET_MGNAME      1
 #define MGPARAMS_SET_SEED        2
@@ -34,19 +34,20 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #define BLOCK_EMERGE_ALLOWGEN (1<<0)
 
 #define EMERGE_DBG_OUT(x) \
-	{ if (enable_mapgen_debug_info) \
-	infostream << "EmergeThread: " x << std::endl; }
+	do {                                                   \
+		if (enable_mapgen_debug_info)                      \
+			infostream << "EmergeThread: " x << std::endl; \
+	} while (0)
 
 class EmergeThread;
-//class Mapgen;
-//struct MapgenFactory;
-class Biome;
-class BiomeDefManager;
-class Decoration;
-class Ore;
 class INodeDefManager;
 class Settings;
 
+class BiomeManager;
+class OreManager;
+class DecorationManager;
+class SchematicManager;
+
 struct BlockMakeData {
 	ManualMapVoxelManipulator *vmanip;
 	u64 seed;
@@ -88,18 +89,21 @@ public:
 	u16 qlimit_diskonly;
 	u16 qlimit_generate;
 
-	u32 gennotify;
+	u32 gen_notify_on;
+	std::set<u32> gen_notify_on_deco_ids;
 
-	//block emerge queue data structures
+	//// Block emerge queue data structures
 	JMutex queuemutex;
 	std::map<v3s16, BlockEmergeData *> blocks_enqueued;
 	std::map<u16, u16> peer_queue_count;
 
-	//Mapgen-related structures
-	BiomeDefManager *biomedef;
-	std::vector<Ore *> ores;
-	std::vector<Decoration *> decorations;
+	//// Managers of map generation-related components
+	BiomeManager *biomemgr;
+	OreManager *oremgr;
+	DecorationManager *decomgr;
+	SchematicManager *schemmgr;
 
+	//// Methods
 	EmergeManager(IGameDef *gamedef);
 	~EmergeManager();
 
@@ -107,7 +111,7 @@ public:
 	void initMapgens();
 	Mapgen *getCurrentMapgen();
 	Mapgen *createMapgen(std::string mgname, int mgid,
-						MapgenParams *mgparams);
+		MapgenParams *mgparams);
 	MapgenSpecificParams *createMapgenParams(std::string mgname);
 	void startThreads();
 	void stopThreads();
