@@ -26,6 +26,8 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #include "threads.h"
 #include "debug.h"
 #include "gettime.h"
+#include "porting.h"
+#include "config.h"
 
 std::list<ILogOutput*> log_outputs[LMT_NUM_VALUES];
 std::map<threadid_t, std::string> log_threadnames;
@@ -58,6 +60,21 @@ void log_remove_output(ILogOutput *out)
 	}
 }
 
+void log_set_lev_silence(enum LogMessageLevel lev, bool silence)
+{
+	log_threadnamemutex.Lock();
+
+	for (std::list<ILogOutput *>::iterator
+			it = log_outputs[lev].begin();
+			it != log_outputs[lev].end();
+			++it) {
+		ILogOutput *out = *it;
+		out->silence = silence;
+	}
+
+	log_threadnamemutex.Unlock();
+}
+
 void log_register_thread(const std::string &name)
 {
 	threadid_t id = get_current_thread_id();
@@ -93,6 +110,7 @@ static std::string get_lev_string(enum LogMessageLevel lev)
 
 void log_printline(enum LogMessageLevel lev, const std::string &text)
 {
+	log_threadnamemutex.Lock();
 	std::string threadname = "(unknown thread)";
 	std::map<threadid_t, std::string>::const_iterator i;
 	i = log_threadnames.find(get_current_thread_id());
@@ -104,10 +122,14 @@ void log_printline(enum LogMessageLevel lev, const std::string &text)
 	for(std::list<ILogOutput*>::iterator i = log_outputs[lev].begin();
 			i != log_outputs[lev].end(); i++){
 		ILogOutput *out = *i;
+		if (out->silence)
+			continue;
+
 		out->printLog(os.str());
 		out->printLog(os.str(), lev);
 		out->printLog(lev, text);
 	}
+	log_threadnamemutex.Unlock();
 }
 
 class Logbuf : public std::streambuf
@@ -137,6 +159,9 @@ public:
 	void printbuf()
 	{
 		log_printline(m_lev, m_buf);
+#ifdef __ANDROID__
+		__android_log_print(ANDROID_LOG_ERROR, PROJECT_NAME, "%s", m_buf.c_str());
+#endif
 	}
 
 	void bufchar(char c)
