@@ -68,7 +68,7 @@ std::string script_get_backtrace(lua_State *L);
 int script_error_handler(lua_State *L);
 int script_exception_wrapper(lua_State *L, lua_CFunction f);
 void script_error(lua_State *L);
-void script_run_callbacks(lua_State *L, int nargs,
-		RunCallbacksMode mode);
+void script_run_callbacks(lua_State *L, int nargs, RunCallbacksMode mode);
+void log_deprecated(lua_State *L, std::string message);
 
 #endif /* C_INTERNAL_H_ */
