@@ -38,7 +38,7 @@ public:
 	virtual ~ScriptApiNode();
 
 	bool node_on_punch(v3s16 p, MapNode node,
-			ServerActiveObject *puncher);
+			ServerActiveObject *puncher, PointedThing pointed);
 	bool node_on_dig(v3s16 p, MapNode node,
 			ServerActiveObject *digger);
 	void node_on_construct(v3s16 p, MapNode node);
