@@ -29,10 +29,7 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 */
 class LuaPerlinNoise : public ModApiBase {
 private:
-	int seed;
-	int octaves;
-	float persistence;
-	float scale;
+	NoiseParams np;
 	static const char className[];
 	static const luaL_reg methods[];
 
@@ -45,16 +42,14 @@ private:
 	static int l_get3d(lua_State *L);
 
 public:
-	LuaPerlinNoise(int a_seed, int a_octaves, float a_persistence,
-			float a_scale);
-
+	LuaPerlinNoise(NoiseParams *params);
 	~LuaPerlinNoise();
 
 	// LuaPerlinNoise(seed, octaves, persistence, scale)
 	// Creates an LuaPerlinNoise and leaves it on top of stack
 	static int create_object(lua_State *L);
 
-	static LuaPerlinNoise* checkobject(lua_State *L, int narg);
+	static LuaPerlinNoise *checkobject(lua_State *L, int narg);
 
 	static void Register(lua_State *L);
 };
@@ -63,8 +58,9 @@ public:
 	LuaPerlinNoiseMap
 */
 class LuaPerlinNoiseMap : public ModApiBase {
-private:
+	NoiseParams np;
 	Noise *noise;
+	bool m_is3d;
 	static const char className[];
 	static const luaL_reg methods[];
 
