@@ -88,6 +88,11 @@ private:
 	// notify_authentication_modified(name)
 	static int l_notify_authentication_modified(lua_State *L);
 
+#ifndef NDEBUG
+	//  cause_error(type_of_error)
+	static int l_cause_error(lua_State *L);
+#endif
+
 public:
 	static void Initialize(lua_State *L, int top);
 
