@@ -166,7 +166,7 @@ struct TestUtilities: public TestBase
 		UASSERT(is_yes("0") == false);
 		UASSERT(is_yes("1") == true);
 		UASSERT(is_yes("2") == true);
-		const char *ends[] = {"abc", "c", "bc", NULL};
+		const char *ends[] = {"abc", "c", "bc", "", NULL};
 		UASSERT(removeStringEnd("abc", ends) == "");
 		UASSERT(removeStringEnd("bc", ends) == "b");
 		UASSERT(removeStringEnd("12c", ends) == "12");
@@ -175,6 +175,40 @@ struct TestUtilities: public TestBase
 				== "%22Aardvarks%20lurk%2C%20OK%3F%22");
 		UASSERT(urldecode("%22Aardvarks%20lurk%2C%20OK%3F%22")
 				== "\"Aardvarks lurk, OK?\"");
+		UASSERT(padStringRight("hello", 8) == "hello   ");
+		UASSERT(str_equal(narrow_to_wide("abc"), narrow_to_wide("abc")));
+		UASSERT(str_equal(narrow_to_wide("ABC"), narrow_to_wide("abc"), true));
+		UASSERT(trim("  a") == "a");
+		UASSERT(trim("   a  ") == "a");
+		UASSERT(trim("a   ") == "a");
+		UASSERT(trim("") == "");
+		UASSERT(mystoi("123", 0, 1000) == 123);
+		UASSERT(mystoi("123", 0, 10) == 10);
+		std::string test_str;
+		test_str = "Hello there";
+		str_replace(test_str, "there", "world");
+		UASSERT(test_str == "Hello world");
+		test_str = "ThisAisAaAtest";
+		str_replace_char(test_str, 'A', ' ');
+		UASSERT(test_str == "This is a test");
+		UASSERT(string_allowed("hello", "abcdefghijklmno") == true);
+		UASSERT(string_allowed("123", "abcdefghijklmno") == false);
+		UASSERT(string_allowed_blacklist("hello", "123") == true);
+		UASSERT(string_allowed_blacklist("hello123", "123") == false);
+		UASSERT(wrap_rows("12345678",4) == "1234\n5678");
+		UASSERT(is_number("123") == true);
+		UASSERT(is_number("") == false);
+		UASSERT(is_number("123a") == false);
+		UASSERT(is_power_of_two(0) == false);
+		UASSERT(is_power_of_two(1) == true);
+		UASSERT(is_power_of_two(2) == true);
+		UASSERT(is_power_of_two(3) == false);
+		for (int exponent = 2; exponent <= 31; ++exponent) {
+			UASSERT(is_power_of_two((1 << exponent) - 1) == false);
+			UASSERT(is_power_of_two((1 << exponent)) == true);
+			UASSERT(is_power_of_two((1 << exponent) + 1) == false);
+		}
+		UASSERT(is_power_of_two((u32)-1) == false);
 	}
 };
 
@@ -239,8 +273,8 @@ struct TestPath: public TestBase
 				expected fs::PathStartsWith results
 				0 = returns false
 				1 = returns true
-				2 = returns false on windows, false elsewhere
-				3 = returns true on windows, true elsewhere
+				2 = returns false on windows, true elsewhere
+				3 = returns true on windows, false elsewhere
 				4 = returns true if and only if
 				    FILESYS_CASE_INSENSITIVE is true
 			*/
@@ -384,27 +418,94 @@ struct TestPath: public TestBase
 	}
 };
 
+#define TEST_CONFIG_TEXT_BEFORE               \
+	"leet = 1337\n"                           \
+	"leetleet = 13371337\n"                   \
+	"leetleet_neg = -13371337\n"              \
+	"floaty_thing = 1.1\n"                    \
+	"stringy_thing = asd /( ¤%&(/\" BLÖÄRP\n" \
+	"coord = (1, 2, 4.5)\n"                   \
+	"      # this is just a comment\n"        \
+	"this is an invalid line\n"               \
+	"asdf = {\n"                              \
+	"	a   = 5\n"                            \
+	"	bb  = 2.5\n"                          \
+	"	ccc = \"\"\"\n"                       \
+	"testy\n"                                 \
+	"   testa   \n"                           \
+	"\"\"\"\n"                                \
+	"\n"                                      \
+	"}\n"                                     \
+	"blarg = \"\"\" \n"                       \
+	"some multiline text\n"                   \
+	"     with leading whitespace!\n"         \
+	"\"\"\"\n"                                \
+	"np_terrain = 5, 40, (250, 250, 250), 12341, 5, 0.7, 2.4\n" \
+	"zoop = true"
+
+#define TEST_CONFIG_TEXT_AFTER                \
+	"leet = 1337\n"                           \
+	"leetleet = 13371337\n"                   \
+	"leetleet_neg = -13371337\n"              \
+	"floaty_thing = 1.1\n"                    \
+	"stringy_thing = asd /( ¤%&(/\" BLÖÄRP\n" \
+	"coord = (1, 2, 4.5)\n"                   \
+	"      # this is just a comment\n"        \
+	"this is an invalid line\n"               \
+	"asdf = {\n"                              \
+	"	a   = 5\n"                            \
+	"	bb  = 2.5\n"                          \
+	"	ccc = \"\"\"\n"                       \
+	"testy\n"                                 \
+	"   testa   \n"                           \
+	"\"\"\"\n"                                \
+	"\n"                                      \
+	"}\n"                                     \
+	"blarg = \"\"\" \n"                       \
+	"some multiline text\n"                   \
+	"     with leading whitespace!\n"         \
+	"\"\"\"\n"                                \
+	"np_terrain = {\n"                        \
+	"	flags = defaults\n"                   \
+	"	lacunarity = 2.4\n"                   \
+	"	octaves = 6\n"                        \
+	"	offset = 3.5\n"                       \
+	"	persistence = 0.7\n"                  \
+	"	scale = 40\n"                         \
+	"	seed = 12341\n"                       \
+	"	spread = (250,250,250)\n"             \
+	"}\n"                                     \
+	"zoop = true\n"                           \
+	"coord2 = (1,2,3.3)\n"                    \
+	"floaty_thing_2 = 1.2\n"                  \
+	"groupy_thing = {\n"                      \
+	"	animals = cute\n"                     \
+	"	num_apples = 4\n"                     \
+	"	num_oranges = 53\n"                   \
+	"}\n"
+
 struct TestSettings: public TestBase
 {
 	void Run()
 	{
+		try {
 		Settings s;
+
 		// Test reading of settings
-		s.parseConfigLine("leet = 1337");
-		s.parseConfigLine("leetleet = 13371337");
-		s.parseConfigLine("leetleet_neg = -13371337");
-		s.parseConfigLine("floaty_thing = 1.1");
-		s.parseConfigLine("stringy_thing = asd /( ¤%&(/\" BLÖÄRP");
-		s.parseConfigLine("coord = (1, 2, 4.5)");
+		std::istringstream is(TEST_CONFIG_TEXT_BEFORE);
+		s.parseConfigLines(is);
+
 		UASSERT(s.getS32("leet") == 1337);
 		UASSERT(s.getS16("leetleet") == 32767);
 		UASSERT(s.getS16("leetleet_neg") == -32768);
+
 		// Not sure if 1.1 is an exact value as a float, but doesn't matter
 		UASSERT(fabs(s.getFloat("floaty_thing") - 1.1) < 0.001);
 		UASSERT(s.get("stringy_thing") == "asd /( ¤%&(/\" BLÖÄRP");
 		UASSERT(fabs(s.getV3F("coord").X - 1.0) < 0.001);
 		UASSERT(fabs(s.getV3F("coord").Y - 2.0) < 0.001);
 		UASSERT(fabs(s.getV3F("coord").Z - 4.5) < 0.001);
+
 		// Test the setting of settings too
 		s.setFloat("floaty_thing_2", 1.2);
 		s.setV3F("coord2", v3f(1, 2, 3.3));
@@ -413,6 +514,69 @@ struct TestSettings: public TestBase
 		UASSERT(fabs(s.getV3F("coord2").X - 1.0) < 0.001);
 		UASSERT(fabs(s.getV3F("coord2").Y - 2.0) < 0.001);
 		UASSERT(fabs(s.getV3F("coord2").Z - 3.3) < 0.001);
+
+		// Test settings groups
+		Settings *group = s.getGroup("asdf");
+		UASSERT(group != NULL);
+		UASSERT(s.getGroupNoEx("zoop", group) == false);
+		UASSERT(group->getS16("a") == 5);
+		UASSERT(fabs(group->getFloat("bb") - 2.5) < 0.001);
+
+		Settings *group3 = new Settings;
+		group3->set("cat", "meow");
+		group3->set("dog", "woof");
+
+		Settings *group2 = new Settings;
+		group2->setS16("num_apples", 4);
+		group2->setS16("num_oranges", 53);
+		group2->setGroup("animals", group3);
+		group2->set("animals", "cute"); //destroys group 3
+		s.setGroup("groupy_thing", group2);
+
+		// Test set failure conditions
+		UASSERT(s.set("Zoop = Poop\nsome_other_setting", "false") == false);
+		UASSERT(s.set("sneaky", "\"\"\"\njabberwocky = false") == false);
+		UASSERT(s.set("hehe", "asdfasdf\n\"\"\"\nsomething = false") == false);
+
+		// Test multiline settings
+		UASSERT(group->get("ccc") == "testy\n   testa   ");
+
+		UASSERT(s.get("blarg") ==
+			"some multiline text\n"
+			"     with leading whitespace!");
+
+		// Test NoiseParams
+		UASSERT(s.getEntry("np_terrain").is_group == false);
+
+		NoiseParams np;
+		UASSERT(s.getNoiseParams("np_terrain", np) == true);
+		UASSERT(fabs(np.offset - 5) < 0.001);
+		UASSERT(fabs(np.scale - 40) < 0.001);
+		UASSERT(fabs(np.spread.X - 250) < 0.001);
+		UASSERT(fabs(np.spread.Y - 250) < 0.001);
+		UASSERT(fabs(np.spread.Z - 250) < 0.001);
+		UASSERT(np.seed == 12341);
+		UASSERT(np.octaves == 5);
+		UASSERT(fabs(np.persist - 0.7) < 0.001);
+
+		np.offset  = 3.5;
+		np.octaves = 6;
+		s.setNoiseParams("np_terrain", np);
+
+		UASSERT(s.getEntry("np_terrain").is_group == true);
+
+		// Test writing
+		std::ostringstream os(std::ios_base::binary);
+		is.clear();
+		is.seekg(0);
+
+		UASSERT(s.updateConfigObject(is, os, "", 0) == true);
+		//printf(">>>> expected config:\n%s\n", TEST_CONFIG_TEXT_AFTER);
+		//printf(">>>> actual config:\n%s\n", os.str().c_str());
+		UASSERT(os.str() == TEST_CONFIG_TEXT_AFTER);
+		} catch (SettingNotFoundException &e) {
+			UASSERT(!"Setting not found!");
+		}
 	}
 };
 
@@ -434,7 +598,7 @@ struct TestSerialization: public TestBase
 		UASSERT(serializeWideString(L"") == mkstr("\0\0"));
 		UASSERT(serializeLongString("") == mkstr("\0\0\0\0"));
 		UASSERT(serializeJsonString("") == "\"\"");
-		
+
 		std::string teststring = "Hello world!";
 		UASSERT(serializeString(teststring) ==
 			mkstr("\0\14Hello world!"));
@@ -560,12 +724,12 @@ struct TestCompress: public TestBase
 		fromdata[1]=5;
 		fromdata[2]=5;
 		fromdata[3]=1;
-		
+
 		std::ostringstream os(std::ios_base::binary);
 		compress(fromdata, os, 0);
 
 		std::string str_out = os.str();
-		
+
 		infostream<<"str_out.size()="<<str_out.size()<<std::endl;
 		infostream<<"TestCompress: 1,5,5,1 -> ";
 		for(u32 i=0; i<str_out.size(); i++)
@@ -616,12 +780,12 @@ struct TestCompress: public TestBase
 		fromdata[1]=5;
 		fromdata[2]=5;
 		fromdata[3]=1;
-		
+
 		std::ostringstream os(std::ios_base::binary);
 		compress(fromdata, os, SER_FMT_VER_HIGHEST_READ);
 
 		std::string str_out = os.str();
-		
+
 		infostream<<"str_out.size()="<<str_out.size()<<std::endl;
 		infostream<<"TestCompress: 1,5,5,1 -> ";
 		for(u32 i=0; i<str_out.size(); i++)
@@ -697,7 +861,7 @@ struct TestMapNode: public TestBase
 		UASSERT(n.getContent() == CONTENT_AIR);
 		UASSERT(n.getLight(LIGHTBANK_DAY, nodedef) == 0);
 		UASSERT(n.getLight(LIGHTBANK_NIGHT, nodedef) == 0);
-		
+
 		// Transparency
 		n.setContent(CONTENT_AIR);
 		UASSERT(nodedef->get(n).light_propagates == true);
@@ -717,28 +881,28 @@ struct TestVoxelManipulator: public TestBase
 		VoxelArea a(v3s16(-1,-1,-1), v3s16(1,1,1));
 		UASSERT(a.index(0,0,0) == 1*3*3 + 1*3 + 1);
 		UASSERT(a.index(-1,-1,-1) == 0);
-		
+
 		VoxelArea c(v3s16(-2,-2,-2), v3s16(2,2,2));
 		// An area that is 1 bigger in x+ and z-
 		VoxelArea d(v3s16(-2,-2,-3), v3s16(3,2,2));
-		
+
 		std::list<VoxelArea> aa;
 		d.diff(c, aa);
-		
+
 		// Correct results
 		std::vector<VoxelArea> results;
 		results.push_back(VoxelArea(v3s16(-2,-2,-3),v3s16(3,2,-3)));
 		results.push_back(VoxelArea(v3s16(3,-2,-2),v3s16(3,2,2)));
 
 		UASSERT(aa.size() == results.size());
-		
+
 		infostream<<"Result of diff:"<<std::endl;
 		for(std::list<VoxelArea>::const_iterator
 				i = aa.begin(); i != aa.end(); ++i)
 		{
 			i->print(infostream);
 			infostream<<std::endl;
-			
+
 			std::vector<VoxelArea>::iterator j = std::find(results.begin(), results.end(), *i);
 			UASSERT(j != results.end());
 			results.erase(j);
@@ -748,13 +912,13 @@ struct TestVoxelManipulator: public TestBase
 		/*
 			VoxelManipulator
 		*/
-		
+
 		VoxelManipulator v;
 
 		v.print(infostream, nodedef);
 
 		infostream<<"*** Setting (-1,0,-1)=2 ***"<<std::endl;
-		
+
 		v.setNodeNoRef(v3s16(-1,0,-1), MapNode(CONTENT_GRASS));
 
 		v.print(infostream, nodedef);
@@ -770,7 +934,7 @@ struct TestVoxelManipulator: public TestBase
 		infostream<<"*** Adding area ***"<<std::endl;
 
 		v.addArea(a);
-		
+
 		v.print(infostream, nodedef);
 
 		UASSERT(v.getNode(v3s16(-1,0,-1)).getContent() == CONTENT_GRASS);
@@ -968,7 +1132,7 @@ struct TestInventory: public TestBase
 		"Empty\n"
 		"EndInventoryList\n"
 		"EndInventory\n";
-		
+
 		std::string serialized_inventory_2 =
 		"List main 32\n"
 		"Width 5\n"
@@ -1006,7 +1170,7 @@ struct TestInventory: public TestBase
 		"Empty\n"
 		"EndInventoryList\n"
 		"EndInventory\n";
-		
+
 		Inventory inv(idef);
 		std::istringstream is(serialized_inventory, std::ios::binary);
 		inv.deSerialize(is);
@@ -1082,7 +1246,7 @@ struct TestMapBlock: public TestBase
 	void Run()
 	{
 		TC parent;
-		
+
 		MapBlock b(&parent, v3s16(1,1,1));
 		v3s16 relpos(MAP_BLOCKSIZE, MAP_BLOCKSIZE, MAP_BLOCKSIZE);
 
@@ -1094,7 +1258,7 @@ struct TestMapBlock: public TestBase
 		UASSERT(b.getBox().MaxEdge.Y == MAP_BLOCKSIZE*2-1);
 		UASSERT(b.getBox().MinEdge.Z == MAP_BLOCKSIZE);
 		UASSERT(b.getBox().MaxEdge.Z == MAP_BLOCKSIZE*2-1);
-		
+
 		UASSERT(b.isValidPosition(v3s16(0,0,0)) == true);
 		UASSERT(b.isValidPosition(v3s16(-1,0,0)) == false);
 		UASSERT(b.isValidPosition(v3s16(-1,-142,-2341)) == false);
@@ -1108,7 +1272,7 @@ struct TestMapBlock: public TestBase
 		*/
 		/*UASSERT(b.getSizeNodes() == v3s16(MAP_BLOCKSIZE,
 				MAP_BLOCKSIZE, MAP_BLOCKSIZE));*/
-		
+
 		// Changed flag should be initially set
 		UASSERT(b.getModified() == MOD_STATE_WRITE_NEEDED);
 		b.resetModified();
@@ -1125,7 +1289,7 @@ struct TestMapBlock: public TestBase
 			UASSERT(b.getNode(v3s16(x,y,z)).getLight(LIGHTBANK_DAY) == 0);
 			UASSERT(b.getNode(v3s16(x,y,z)).getLight(LIGHTBANK_NIGHT) == 0);
 		}
-		
+
 		{
 			MapNode n(CONTENT_AIR);
 			for(u16 z=0; z<MAP_BLOCKSIZE; z++)
@@ -1135,7 +1299,7 @@ struct TestMapBlock: public TestBase
 				b.setNode(v3s16(x,y,z), n);
 			}
 		}
-			
+
 		/*
 			Parent fetch functions
 		*/
@@ -1143,7 +1307,7 @@ struct TestMapBlock: public TestBase
 		parent.node.setContent(5);
 
 		MapNode n;
-		
+
 		// Positions in the block should still be valid
 		UASSERT(b.isValidPositionParent(v3s16(0,0,0)) == true);
 		UASSERT(b.isValidPositionParent(v3s16(MAP_BLOCKSIZE-1,MAP_BLOCKSIZE-1,MAP_BLOCKSIZE-1)) == true);
@@ -1154,7 +1318,7 @@ struct TestMapBlock: public TestBase
 		UASSERT(b.isValidPositionParent(v3s16(-121,2341,0)) == false);
 		UASSERT(b.isValidPositionParent(v3s16(-1,0,0)) == false);
 		UASSERT(b.isValidPositionParent(v3s16(MAP_BLOCKSIZE-1,MAP_BLOCKSIZE-1,MAP_BLOCKSIZE)) == false);
-		
+
 		{
 			bool exception_thrown = false;
 			try{
@@ -1186,7 +1350,7 @@ struct TestMapBlock: public TestBase
 		//TODO: Update to new system
 		/*UASSERT(b.getNodeTile(p) == 4);
 		UASSERT(b.getNodeTile(v3s16(-1,-1,0)) == 5);*/
-		
+
 		/*
 			propagateSunlight()
 		*/
@@ -1316,29 +1480,29 @@ struct TestMapSector: public TestBase
 			if(position_valid == false)
 				throw InvalidPositionException();
 		};
-		
+
 		virtual u16 nodeContainerId() const
 		{
 			return 666;
 		}
 	};
-	
+
 	void Run()
 	{
 		TC parent;
 		parent.position_valid = false;
-		
+
 		// Create one with no heightmaps
 		ServerMapSector sector(&parent, v2s16(1,1));
-		
+
 		UASSERT(sector.getBlockNoCreateNoEx(0) == 0);
 		UASSERT(sector.getBlockNoCreateNoEx(1) == 0);
 
 		MapBlock * bref = sector.createBlankBlock(-2);
-		
+
 		UASSERT(sector.getBlockNoCreateNoEx(0) == 0);
 		UASSERT(sector.getBlockNoCreateNoEx(-2) == bref);
-		
+
 		//TODO: Check for AlreadyExistsException
 
 		/*bool exception_thrown = false;
@@ -1511,35 +1675,46 @@ struct TestSocket: public TestBase
 
 		// IPv6 socket test
 		{
-			UDPSocket socket6(true);
-			socket6.Bind(address6);
+			UDPSocket socket6;
 
-			const char sendbuffer[] = "hello world!";
-			IPv6AddressBytes bytes;
-			bytes.bytes[15] = 1;
-			
-			try {
-				socket6.Send(Address(&bytes, port), sendbuffer, sizeof(sendbuffer));
+			if (!socket6.init(true, true)) {
+				/* Note: Failing to create an IPv6 socket is not technically an
+				   error because the OS may not support IPv6 or it may
+				   have been disabled. IPv6 is not /required/ by
+				   minetest and therefore this should not cause the unit
+				   test to fail
+				*/
+				dstream << "WARNING: IPv6 socket creation failed (unit test)"
+				        << std::endl;
+			} else {
+				const char sendbuffer[] = "hello world!";
+				IPv6AddressBytes bytes;
+				bytes.bytes[15] = 1;
 
-				sleep_ms(50);
+				socket6.Bind(address6);
 
-				char rcvbuffer[256];
-				memset(rcvbuffer, 0, sizeof(rcvbuffer));
-				Address sender;
-				for(;;)
-				{
-					int bytes_read = socket6.Receive(sender, rcvbuffer, sizeof(rcvbuffer));
-					if(bytes_read < 0)
-						break;
+				try {
+					socket6.Send(Address(&bytes, port), sendbuffer, sizeof(sendbuffer));
+
+					sleep_ms(50);
+
+					char rcvbuffer[256] = { 0 };
+					Address sender;
+
+					for(;;) {
+						if (socket6.Receive(sender, rcvbuffer, sizeof(rcvbuffer )) < 0)
+							break;
+					}
+					//FIXME: This fails on some systems
+					UASSERT(strncmp(sendbuffer, rcvbuffer, sizeof(sendbuffer)) == 0);
+					UASSERT(memcmp(sender.getAddress6().sin6_addr.s6_addr,
+							Address(&bytes, 0).getAddress6().sin6_addr.s6_addr, 16) == 0);
+				}
+				catch (SendFailedException e) {
+					errorstream << "IPv6 support enabled but not available!"
+					            << std::endl;
 				}
-				//FIXME: This fails on some systems
-				UASSERT(strncmp(sendbuffer, rcvbuffer, sizeof(sendbuffer))==0);
-				UASSERT(memcmp(sender.getAddress6().sin6_addr.s6_addr, Address(&bytes, 0).getAddress6().sin6_addr.s6_addr, 16) == 0);
 			}
-			catch (SendFailedException e) {
-				errorstream << "IPv6 support enabled but not available!" << std::endl;
- 			}
-			
 		}
 
 		// IPv4 socket test
@@ -1548,22 +1723,20 @@ struct TestSocket: public TestBase
 			socket.Bind(address);
 
 			const char sendbuffer[] = "hello world!";
-			socket.Send(Address(127,0,0,1,port), sendbuffer, sizeof(sendbuffer));
+			socket.Send(Address(127, 0, 0 ,1, port), sendbuffer, sizeof(sendbuffer));
 
 			sleep_ms(50);
 
-			char rcvbuffer[256];
-			memset(rcvbuffer, 0, sizeof(rcvbuffer));
+			char rcvbuffer[256] = { 0 };
 			Address sender;
-			for(;;)
-			{
-				int bytes_read = socket.Receive(sender, rcvbuffer, sizeof(rcvbuffer));
-				if(bytes_read < 0)
+			for(;;) {
+				if (socket.Receive(sender, rcvbuffer, sizeof(rcvbuffer)) < 0)
 					break;
 			}
 			//FIXME: This fails on some systems
-			UASSERT(strncmp(sendbuffer, rcvbuffer, sizeof(sendbuffer))==0);
-			UASSERT(sender.getAddress().sin_addr.s_addr == Address(127,0,0,1, 0).getAddress().sin_addr.s_addr);
+			UASSERT(strncmp(sendbuffer, rcvbuffer, sizeof(sendbuffer)) == 0);
+			UASSERT(sender.getAddress().sin_addr.s_addr ==
+					Address(127, 0, 0, 1, 0).getAddress().sin_addr.s_addr);
 		}
 	}
 };
@@ -1583,7 +1756,7 @@ struct TestConnection: public TestBase
 		SharedBuffer<u8> data1(1);
 		data1[0] = 100;
 		Address a(127,0,0,1, 10);
-		u16 seqnum = 34352;
+		const u16 seqnum = 34352;
 
 		con::BufferedPacket p1 = con::makePacket(a, data1,
 				proto_id, peer_id, channel);
@@ -1600,7 +1773,7 @@ struct TestConnection: public TestBase
 		UASSERT(readU16(&p1.data[4]) == peer_id);
 		UASSERT(readU8(&p1.data[6]) == channel);
 		UASSERT(readU8(&p1.data[7]) == data1[0]);
-		
+
 		//infostream<<"initial data1[0]="<<((u32)data1[0]&0xff)<<std::endl;
 
 		SharedBuffer<u8> p2 = con::makeReliablePacket(data1, seqnum);
@@ -1662,29 +1835,29 @@ struct TestConnection: public TestBase
 
 		Handler hand_server("server");
 		Handler hand_client("client");
-		
+
 		infostream<<"** Creating server Connection"<<std::endl;
 		con::Connection server(proto_id, 512, 5.0, false, &hand_server);
 		Address address(0,0,0,0, 30001);
 		server.Serve(address);
-		
+
 		infostream<<"** Creating client Connection"<<std::endl;
 		con::Connection client(proto_id, 512, 5.0, false, &hand_client);
-		
+
 		UASSERT(hand_server.count == 0);
 		UASSERT(hand_client.count == 0);
-		
+
 		sleep_ms(50);
-		
+
 		Address server_address(127,0,0,1, 30001);
 		infostream<<"** running client.Connect()"<<std::endl;
 		client.Connect(server_address);
 
 		sleep_ms(50);
-		
+
 		// Client should not have added client yet
 		UASSERT(hand_client.count == 0);
-		
+
 		try
 		{
 			u16 peer_id;
@@ -1704,7 +1877,7 @@ struct TestConnection: public TestBase
 		UASSERT(hand_client.last_id == 1);
 		// Server should not have added client yet
 		UASSERT(hand_server.count == 0);
-		
+
 		sleep_ms(100);
 
 		try
@@ -1722,14 +1895,14 @@ struct TestConnection: public TestBase
 			// No actual data received, but the client has
 			// probably been connected
 		}
-		
+
 		// Client should be the same
 		UASSERT(hand_client.count == 1);
 		UASSERT(hand_client.last_id == 1);
 		// Server should have the client
 		UASSERT(hand_server.count == 1);
 		UASSERT(hand_server.last_id == 2);
-		
+
 		//sleep_ms(50);
 
 		while(client.Connected() == false)
@@ -1751,7 +1924,7 @@ struct TestConnection: public TestBase
 		}
 
 		sleep_ms(50);
-		
+
 		try
 		{
 			u16 peer_id;
@@ -1804,10 +1977,10 @@ struct TestConnection: public TestBase
 
 			Address client_address =
 					server.GetPeerAddress(peer_id_client);
-			
+
 			infostream<<"*** Sending packets in wrong order (2,1,2)"
 					<<std::endl;
-			
+
 			u8 chn = 0;
 			con::Channel *ch = &server.getPeer(peer_id_client)->channels[chn];
 			u16 sn = ch->next_outgoing_seqnum;
@@ -1836,7 +2009,7 @@ struct TestConnection: public TestBase
 			UASSERT(size == data1.getSize());
 			UASSERT(memcmp(*data1, *recvdata, data1.getSize()) == 0);
 			UASSERT(peer_id == PEER_ID_SERVER);
-			
+
 			infostream<<"** running client.Receive()"<<std::endl;
 			peer_id = 132;
 			size = client.Receive(peer_id, recvdata);
@@ -1847,7 +2020,7 @@ struct TestConnection: public TestBase
 			UASSERT(size == data2.getSize());
 			UASSERT(memcmp(*data2, *recvdata, data2.getSize()) == 0);
 			UASSERT(peer_id == PEER_ID_SERVER);
-			
+
 			bool got_exception = false;
 			try
 			{
@@ -1881,14 +2054,14 @@ struct TestConnection: public TestBase
 				SharedBuffer<u8> data1(datasize);
 				for(u16 i=0; i<datasize; i++)
 					data1[i] = i/4;
-				
+
 				int sendtimes = myrand_range(1,10);
 				for(int i=0; i<sendtimes; i++){
 					server.Send(peer_id_client, 0, data1, true);
 					sendcount++;
 				}
 				infostream<<"sendcount="<<sendcount<<std::endl;
-				
+
 				//int receivetimes = myrand_range(1,20);
 				int receivetimes = 20;
 				for(int i=0; i<receivetimes; i++){
@@ -1925,11 +2098,11 @@ struct TestConnection: public TestBase
 			if(datasize>20)
 				infostream<<"...";
 			infostream<<std::endl;
-			
+
 			server.Send(peer_id_client, 0, data1, true);
 
 			//sleep_ms(3000);
-			
+
 			SharedBuffer<u8> recvdata;
 			infostream<<"** running client.Receive()"<<std::endl;
 			u16 peer_id = 132;
@@ -1965,34 +2138,32 @@ struct TestConnection: public TestBase
 			UASSERT(memcmp(*data1, *recvdata, data1.getSize()) == 0);
 			UASSERT(peer_id == PEER_ID_SERVER);
 		}
-		
+
 		// Check peer handlers
 		UASSERT(hand_client.count == 1);
 		UASSERT(hand_client.last_id == 1);
 		UASSERT(hand_server.count == 1);
 		UASSERT(hand_server.last_id == 2);
-		
+
 		//assert(0);
 	}
 };
 
-#define TEST(X)\
-{\
+#define TEST(X) do {\
 	X x;\
 	infostream<<"Running " #X <<std::endl;\
 	x.Run();\
 	tests_run++;\
 	tests_failed += x.test_failed ? 1 : 0;\
-}
+} while (0)
 
-#define TESTPARAMS(X, ...)\
-{\
+#define TESTPARAMS(X, ...) do {\
 	X x;\
 	infostream<<"Running " #X <<std::endl;\
 	x.Run(__VA_ARGS__);\
 	tests_run++;\
 	tests_failed += x.test_failed ? 1 : 0;\
-}
+} while (0)
 
 void run_tests()
 {
@@ -2000,12 +2171,14 @@ void run_tests()
 
 	int tests_run = 0;
 	int tests_failed = 0;
-	
+
 	// Create item and node definitions
 	IWritableItemDefManager *idef = createItemDefManager();
 	IWritableNodeDefManager *ndef = createNodeDefManager();
 	define_some_nodes(idef, ndef);
 
+	log_set_lev_silence(LMT_ERROR, true);
+
 	infostream<<"run_tests() started"<<std::endl;
 	TEST(TestUtilities);
 	TEST(TestPath);
@@ -2027,6 +2200,8 @@ void run_tests()
 		dout_con<<"=== END RUNNING UNIT TESTS FOR CONNECTION ==="<<std::endl;
 	}
 
+	log_set_lev_silence(LMT_ERROR, false);
+
 	delete idef;
 	delete ndef;
 
